"""NDG XACML Context Environment type

NERC DataGrid
"""
__author__ = "P J Kershaw"
__date__ = "24/03/10"
__copyright__ = "(C) 2010 Science and Technology Facilities Council"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id: environment.py 7087 2010-06-25 11:23:09Z pjkersha $"
from ndg.xacml.core.context import RequestChildBase


class Environment(RequestChildBase):
    """XACML Context Environment type"""
    ELEMENT_LOCAL_NAME = 'Environment'

